/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.trainexample;

/**
 * This class represents CargoCarriage.
 *
 * Created 12.11.2009 at 18:38:27.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class CargoCarriage extends Carriage{

    private double cargoCapacity; // in tons
    private double actualCargo = 0; // in tons

    /**
     * Constructs new cargo carriage
     * @param id id of rail vehicle used by railways to identify vehicle
     * @param length of carriage in meters
     * @param cargoCapacity maximal cargo capacity of cargo carriage in tons
     * @see Carriage#Carriage(java.lang.String, double) for details on length and id
     * @throws IllegalArgumentException if cargoCapacity <= 0
     */
    public CargoCarriage(String id, double length, double cargoCapacity) {
        super(id, length);
        if(cargoCapacity <= 0){
            throw new IllegalArgumentException("Cargo capacity must be greater " +
                    "then zero but was " + cargoCapacity);
        }
        this.cargoCapacity = cargoCapacity;
    }

    /**
     * Standard getter for actualCargo field
     * @return actual cargo of this cargo carriage in tons
     */
    public double getActualCargo() {
        return actualCargo;
    }

    /**
     * Standard getter for cargoCapacity field
     * @return maximal cargo capacity of this cargo carriage in tons
     */
    public double getCargoCapacity() {
        return cargoCapacity;
    }

    /**
     * Standard setter for actualCargofield
     * @param actualCargo new actual cargo of this cargo carriage in tons
     * @throws IllegalArgumentException if actualCargo < 0 or actualCargo > {@link CargoCarriage#getCargoCapacity()}
     */
    public void setActualCargo(double actualCargo) {
        if(actualCargo > getCargoCapacity()){
            throw new IllegalArgumentException("Given cargo is greater then maximal " +
                    "cargo capacity of this carriage. Capacity is " + getCargoCapacity() +
                    " but cargo was " + actualCargo);
        }
        if(actualCargo < 0){
            throw new IllegalArgumentException("Cagro cannot be of negative mass but was " + actualCargo);
        }
        this.actualCargo = actualCargo;
    }

    /**
     * Holds overall contract of method equals (see {@link Object#equals(java.lang.Object)}).
     * Given object is equal to this cargo carriage if and only if is CargoCarriage and
     * it has equal id to id of this cargo carriage.
     *
     * NOTE: demostrates proper implementation of equals. When you override equals you must always
     * override {@link Object#hashCode()} too. Uses key word <code>instanceof</code> and type changing.
     * For other examples on use ot this keyword see {@link Train#getPassangersCount()} and {@link Train#getCargoCount()}
     * methods.
     *
     * @param obj object in task
     * @return true if and only if this cargo carriage is equal to given object, false else
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        //check if obj is instance of CargoCarriage
        if(!(obj instanceof CargoCarriage)){
            //if obj is not instance of CargoCarriage it is not equal to this CargoCarriage. Return false
            return false;
        }

        //obj is instance of CargoCarriage. Change its type to CargoCarriage
        CargoCarriage other = (CargoCarriage) obj;
        //direct access to atributes of other object!!! Recomended only in this case(method equals).
        return getId().equals(other.getId());
    }

    /**
     * Return the number that is always the same for equal cargo carriage,
     * in most cases schould be different for not equals cargo carriage but sometimes may be same.
     * That is in the other words contract of method hashCode for this class.
     * For more details on overall constract see {@link Object#hashCode()}
     *
     * @return hash code value for this object.
     */
    @Override
    public int hashCode() {
        return getId().hashCode();
    }

    @Override
    public String toString() {
        return "Cargo " + super.toString() + " and maximum cargo capacity " + cargoCapacity + " tons." +
                "Current cargo load is " + actualCargo;
    }




}
