/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.trainexample;

/**
 * This class represents Locomotive.
 *
 * Created 12.11.2009 at 18:38:02.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class Locomotive extends RailVehicle{

    private double power; //in kW

    /**
     * Constructs new Locomotive
     *
     * @param id id of locomotive used by railwais
     * @param power power of locomotive in kW
     * @see RailVehicle#RailVehicle(java.lang.String) for detail on id value
     */
    public Locomotive(String id, double power) {
        super(id);
        if(power <= 0){
            throw new IllegalArgumentException("power of locomotive must be " +
                    "greater then zero but was" + power);
        }
        this.power = power;
    }

    /**
     * Standard getter for field power
     * @return Power of locomotive in kW
     */
    public double getPower() {
        return power;
    }

    /**
     * Holds overall contract of method equals (see {@link Object#equals(java.lang.Object)}).
     * Given object is equal to this locomotive if and only if is Locomotive and
     * it has equal id to id of this locomotive.
     *
     * NOTE: demostrates proper implementation of equals. When you override equals you must always
     * override {@link Object#hashCode()} too. Uses key word <code>instanceof</code> and type changing.
     * For other examples on use ot this keyword see {@link Train#getPassangersCount()} and {@link Train#getCargoCount()}
     * methods.
     *
     * @param obj object in task
     * @return true if and only if this locomotive is equal to given object, false else
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        //check if obj is instance of Locomotive
        if(!(obj instanceof Locomotive)){
            //if obj is not instance of Locomotive it is not equal to this Locomotive. Return false
            return false;
        }

        //obj is instance of Locomotive. Change its type to Locomotive
        Locomotive other = (Locomotive) obj;
        //direct access to atributes of other object!!! Recomended only in this case(method equals).
        return getId().equals(other.getId());
    }

    /**
     * Return the number that is always the same for equal locomotives,
     * in most cases schould be different for not equals locomotives but sometimes may be same.
     * That is in the other words contract of method hashCode for this class.
     * For more details on overall constract see {@link Object#hashCode()}
     *
     * @return hash code value for this object.
     */
    @Override
    public int hashCode() {
        return getId().hashCode();
    }

    @Override
    public String toString() {
        return "Locomotive with id " + getId() + " and power " + power;
    }





}
