/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.trainexample;

/**
 * This class represents PersonalCarriage.
 *
 * Created 12.11.2009 at 18:38:38.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class PersonalCarriage extends Carriage{

    private int chairCount;
    private int passangerCount;

    /**
     * Constructs new personal carriage
     * @param id id of rail vehicle used by railways to identify vehicle
     * @param length of carriage in meters
     * @param chairCount number of chairs in carriage
     * @see Carriage#Carriage(java.lang.String, double) for details on length and id
     * @throws IllegalArgumentException if chairCount <= 0
     */
    public PersonalCarriage(String id, double length, int chairCount) {
        super(id, length);
        if(chairCount <= 0){
            throw new IllegalArgumentException("chair count must be greater then zero but was " + chairCount);
        }
        this.chairCount = chairCount;
    }

    /**
     * Standard getter for chairCount field
     * @return number of chairs in carriage
     */
    public int getChairCount() {
        return chairCount;
    }

    /**
     * Standard getter for passangerCount field
     * @return number of passangers currently in carriage
     */
    public int getPassangerCount() {
        return passangerCount;
    }

    /**
     * Standard setter for passangerCount field.
     * It is allowed for the passangerCount parameter to be greater then chairCount.
     * Some passengers can stand in the corridor.
     * @param passangerCount number of passangers that are in this carriage
     * @throws IllegalArgumentException if passangerCount is less then zero
     */
    public void setPassangerCount(int passangerCount) {
        if(passangerCount < 0){
            throw new IllegalArgumentException("passangerCount must be greater " +
                    "then zero but was " + passangerCount);
        }
        this.passangerCount = passangerCount;
    }

    /**
     * Holds overall contract of method equals (see {@link Object#equals(java.lang.Object)}).
     * Given object is equal to this personal carriage if and only if is PersonalCarriage and
     * it has equal id to id of this personal carriage.
     *
     * NOTE: demostrates proper implementation of equals. When you override equals you must always
     * override {@link Object#hashCode()} too. Uses key word <code>instanceof</code> and type changing.
     * For other examples on use ot this keyword see {@link Train#getPassangersCount()} and {@link Train#getCargoCount()}
     * methods.
     *
     * @param obj object in task
     * @return true if and only if this personal carriage is equal to given object, false else
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        //check if obj is instance of PersonalCarriage
        if(!(obj instanceof PersonalCarriage)){
            //if obj is not instance of PersonalCarriage it is not equal to this PersonalCarriage. Return false
            return false;
        }

        //obj is instance of PersonalCarriage. Change its type to PersonalCarriage
        PersonalCarriage other = (PersonalCarriage) obj;
        //direct access to atributes of other object!!! Recomended only in this case(method equals).
        return getId().equals(other.getId());
    }

    /**
     * Return the number that is always the same for equal personal carriage,
     * in most cases schould be different for not equals personal carriage but sometimes may be same.
     * That is in the other words contract of method hashCode for this class.
     * For more details on overall constract see {@link Object#hashCode()}
     *
     * @return hash code value for this object.
     */
    @Override
    public int hashCode() {
        return getId().hashCode();
    }

    @Override
    public String toString() {
        return "Personal " + super.toString() + " and " + chairCount + " seats. In the moment there are " +
                passangerCount + " passangers in the carriage.";
    }


}
